from fastapi import APIRouter, Depends
from fastapi.responses import JSONResponse

from bsition.api.models.document import Access
from bsition.api.models.table import Filter, Sort, Table
from bsition.api.utils.security import get_current_user
from bsition.backend.mongo import tables as mongo
from bsition.backend.postgres import tables as postgres_t
from bsition.backend.postgres import relations as postgres_r

router = APIRouter()


@router.post("")
def create_table(aux: Table):
    mongo.create_table(aux.name)
    return JSONResponse(content={"detail": "Table created."}, status_code=201)


@router.put("/{name}")
def edit_table(aux: Table, name: str):
    if aux.column is not None and aux.type is not None:
        mongo.add_column(name, aux.column, aux.type)
    if aux.column_data is not None:
        mongo.insert_columns(name, aux.column_data)
    if aux.row_number is not None:
        mongo.edit_columns(name, aux.columns, aux.columns_data, aux.row_number)
    return JSONResponse(content={"detail": "Table updated."}, status_code=202)


@router.post("/{name}/sort")
def create_sort(name: str):
    postgres_t.create_sort()
    return JSONResponse(content={"detail": "Sort created."}, status_code=201)


@router.put("/{name}/sort")
def add_sort(aux: Sort, name: str):
    postgres_t.add_sort(name, aux.property, aux.order, aux.priority)
    return JSONResponse(content={"detail": "Sort updated."}, status_code=202)


@router.get("/{name}/sort")
def sort(name: str):
    return mongo.sort(name)


@router.post("/{name}/filter")
def create_filter(name: str):
    postgres_t.create_filter()
    return JSONResponse(content={"detail": "Filter created."}, status_code=201)


@router.put("/{name}/filter")
def add_filter(aux: Filter, name: str):
    postgres_t.add_filter(name, aux.property, aux.value, aux.function)
    return JSONResponse(content={"detail": "Filter updated."}, status_code=202)


@router.get("/{name}/filter")
def filter(name: str):
    return mongo.filter(name)


@router.get("/{id}/access")
def get_access(id: str, user: tuple = Depends(get_current_user)):
    return postgres_r.has_access_table(user[0], id)


@router.post("/{id}/access")
def give_access(id: str, access_type: Access, user: tuple = Depends(get_current_user)):
    postgres_r.give_access_table(user[0], id, access_type.access_type)
    return JSONResponse(content={"detail": "Access created"}, status_code=201)


@router.put("/{id}/access")
def update_access(id: str, access_type: Access, user: tuple = Depends(get_current_user)):
    postgres_r.give_access_table(user[0], id, access_type.access_type)
    return JSONResponse(content={"detail": "Access updated"}, status_code=202)


@router.delete("/{id}/access")
def deny_access(id: str, user: tuple = Depends(get_current_user)):
    postgres_r.deny_access_table(user[0], id)
    return JSONResponse(content={"detail": "Access deleted"}, status_code=201)
